const std = @import("std");

const CellStatus = enum(u2) {
    Rock = 0,
    Reachable = 1,
    ReachedEven = 2,
    ReachedOdd = 3,
};

const Map = [256][256]CellStatus;

fn solveMap(map: Map) usize {
    const height = map.len;
    const width = map[0].len;
    var current_map = map;
    for (0..64) |i| {
        _ = i;
        var next_map = current_map;
        for (0..height) |row| {
            for (0..width) |column| {
                const maybe_next_status: ?CellStatus = switch (current_map[row][column]) {
                    .ReachedEven => .ReachedOdd,
                    .ReachedOdd => .ReachedEven,
                    else => null,
                };

                if (maybe_next_status) |next_status| {
                    if (row > 0 and next_map[row - 1][column] == .Reachable) {
                        next_map[row - 1][column] = next_status;
                    }

                    if (row + 1 < height and next_map[row + 1][column] == .Reachable) {
                        next_map[row + 1][column] = next_status;
                    }

                    if (column > 0 and next_map[row][column - 1] == .Reachable) {
                        next_map[row][column - 1] = next_status;
                    }

                    if (column + 1 < width and next_map[row][column + 1] == .Reachable) {
                        next_map[row][column + 1] = next_status;
                    }
                }
            }
        }

        current_map = next_map;
    }

    var result: usize = 0;
    for (0..height) |row| {
        for (0..width) |column| {
            if (current_map[row][column] == .ReachedEven) {
                result += 1;
            }
        }
    }

    return result;
}

pub fn solveAll(reader: anytype) !usize {
    var result: usize = 0;
    while (true) {
        var map = std.mem.zeroes(Map);

        var row: usize = 0;
        var empty_line_reached = false;
        var line_buffer: [1000]u8 = undefined;
        while (try reader.readUntilDelimiterOrEof(&line_buffer, '\n')) |line| {
            if (line.len == 0) {
                empty_line_reached = true;
                break;
            }
            for (line, 0..) |char, column| {
                map[row][column] = switch (char) {
                    '#' => .Rock,
                    '.' => .Reachable,
                    'S' => .ReachedEven,
                    else => unreachable,
                };
            }
            row += 1;
        }

        result += solveMap(map);

        if (!empty_line_reached) {
            return result;
        }
    }
}

pub fn main() !void {
    const stdout = std.io.getStdOut().writer();

    const raw_in = std.io.getStdIn();
    var buffered_reader = std.io.bufferedReader(raw_in.reader());
    var reader = buffered_reader.reader();
    const result = try solveAll(&reader);
    try stdout.print("{d}\n", .{result});
}
